@@ -2,12 +2,32 @@ module RSpec
   module Core
     module SharedExampleGroup
 
-      def share_examples_for(name, &block)
-        ensure_shared_example_group_name_not_taken(name)
-        RSpec.world.shared_example_groups[name] = block
+      # @overload shared_examples(name, &block)
+      # @overload shared_examples(name, tags, &block)
+      #
+      # Creates and stores (but does not evaluate) the block.
+      #
+      # @see ExampleGroup.include_examples
+      # @see ExampleGroup.include_context
+      def shared_examples(*args, &block)
+        if [String, Symbol, Module].any? {|cls| cls === args.first }
+          object = args.shift
+          ensure_shared_example_group_name_not_taken(object)
+          RSpec.world.shared_example_groups[object] = block
+        end
+
+        unless args.empty?
+          mod = Module.new
+          (class << mod; self; end).send(:define_method, :extended) do |host|
+            host.class_eval(&block)
+          end
+          RSpec.configuration.extend(mod, *args)
+        end
       end
 
-      alias :shared_examples_for :share_examples_for
+      alias_method :shared_context,      :shared_examples
+      alias_method :share_examples_for,  :shared_examples
+      alias_method :shared_examples_for, :shared_examples
 
       def share_as(name, &block)
         if Object.const_defined?(name)
@@ -33,7 +53,6 @@ module RSpec
         RSpec.world.shared_example_groups[shared_const] = block
       end
 
-
     private
 
       def raise_name_error
